package mmtext;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.imageio.ImageIO;

import mmtext.images.BinaryImage;
import mmtext.images.GrayScaleImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IFloatImage;
import mmtext.images.IGrayScaleImage;
import mmtext.images.operators.MorphologicalOperator;
import mmtext.images.operators.ThresholdGlobal;
import mmtext.images.operators.ThresholdLocal;
import mmtext.images.operators.uao.MaxtreeBuilder;
import mmtext.images.operators.uao.UltimateOpening;
import mmtext.images.utils.ImageBuilder;
import mmtext.images.utils.ImageUtils;

public class MMTextBinarize {

	public static void main(String[] args) throws Exception {
		double f = 0;
		double r= 0;
		double p=0;
		int n=0;
		double PSNR = 0;
		double NRM = 0;
		File pathsR1[] = new File("img/DIBC02009_Test_images-handwritten").listFiles();
		File pathsC1[] = new File("img/DIBCO2009-GT-Test-images_handwritten").listFiles();

		File pathsR2[] = new File("img/DIBCO2009_Test_images-printed").listFiles();
		File pathsC2[] = new File("img/DIBCO2009-GT-Test-images_printed").listFiles();

		File pathsR3[] = new File[]{};//new File("img/H_DIBCO2010_test_images").listFiles();
		File pathsC3[] = new File[]{};//new File("img/H_DIBCO2010_GT").listFiles();

		
		File pathsR[] = new File[pathsR1.length + pathsR2.length + pathsR3.length];
		File pathsC[] = new File[pathsC1.length + pathsC2.length + pathsC3.length];

		for(int i=0; i < pathsR.length; i++){
			if(i < pathsR1.length)
				pathsR[i] = pathsR1[i];
			else if( i < pathsR1.length + pathsR2.length)
				pathsR[i] = pathsR2[i - pathsR1.length];
			else 
				pathsR[i] = pathsR3[i - (pathsR1.length + pathsR2.length)];
		}
		for(int i=0; i < pathsC.length; i++){
			if(i < pathsC1.length)
				pathsC[i] = pathsC1[i];
			else if(i < pathsC1.length + pathsC2.length)
				pathsC[i] = pathsC2[i - pathsC1.length];
			else 
				pathsC[i] = pathsC3[i - (pathsC1.length + pathsC2.length)]; 
		}
		
		for (int ii = 0; ii < pathsR.length; ii++)
			for (int jj = 0; jj < pathsC.length; jj++)
				if (pathsR[ii].getName().charAt(2) == pathsC[jj].getName().charAt(2) && pathsR[ii].getName().charAt(0) == pathsC[jj].getName().charAt(0)) {
					
					
					
					String pathR = pathsR[ii].getPath();
					String pathC = pathsC[jj].getPath();
					System.out.println("\nResult: "+ pathR);
					System.out.println("Correct: " + pathC);
					IBinaryImage imgResult = null;

					
					//imgResult = HDILocalMaxMin.getBinary(new GrayScaleImage(ImageBuilder.convertToArray(ImageIO.read(new File(pathR)))));
					IGrayScaleImage img = new GrayScaleImage(ImageBuilder.convertToArray(ImageIO.read(new File(pathR))));
					imgResult = getBinary(img, img.getHeight(), 2);
					
					
					
					 IBinaryImage se = MorphologicalOperator.createImageDecorator(255, 0, 5,5);
				        
				        IFloatImage Emean = HDILocalMaxMin.getMean(img, imgResult, se);
				        IFloatImage Estd = HDILocalMaxMin.getStandardDeviation(img, imgResult, Emean, se);
				        
				        int nMin = se.getArea()/12;
				        
				        for(int w=0; w < img.getWidth(); w++){
				        	for(int h=0; h < img.getHeight(); h++){
				        		int ne = 0;
				        		for(int x=-se.getWidth()/2, xx=0; xx < se.getWidth(); x++, xx++){ //Varendo o elemento estruturante
				                    for(int y=-se.getHeight()/2, yy=0; yy < se.getHeight(); y++, yy++){
				                        if(ImageUtils.isIndexInImage(img, w + xx, h + yy) && !se.isPixelBackground(xx, yy)){ 
				                        	  if(!imgResult.isPixelBackground(xx + w, yy + h)){
				                        		  ne++;
				                        	  }
				                       }
				                    }
				        		}
				        		
				        		if(ne >= nMin && img.getPixel(w, h) <= Emean.getPixel(w, h) + Estd.getPixel(w, h)/2){
				        			imgResult.setPixel(w, h, 0);
				        		}
				        			
				        	}
				        }
				    imgResult = MorphologicalOperator.areaOpen(imgResult, 20);
				    
					IBinaryImage imgCorrect = new BinaryImage(ImageBuilder.convertToArray(ImageIO.read(new File(pathC))));
					int cTP = 0;
					int cFP = 0;
					int cFN = 0;
					int cTN = 0;
					double mse = 0;
					double c = 0;
					for (int i = 0; i < imgResult.getSize(); i++) {
						if (imgResult.getPixel(i) == 0	&& imgCorrect.getPixel(i) == 0) {
							cTP++;
						} else if (imgResult.getPixel(i) == 0 && imgCorrect.getPixel(i) == 255) {
							cFP++;
						} else if (imgResult.getPixel(i) == 255 && imgCorrect.getPixel(i) == 0) {
							cFN++;
						}else{
							cTN++;
						}
						
						
					}
					
					double recall = cTP / (double) (cFN + cTP);
					double precision = cTP / (double) (cFP + cTP);
					double fm = (2 * recall * precision / (recall + precision)) * 100;
					double psnr = 10 * ImageUtils.log10(imgResult.getSize() / (cFP + cFN));
					
					
					double nrfn = cFN / (double)(cFN + cTP);
					double nrfp = cFP / (double)(cFP + cTN);
					double nrm = (nrfn + nrfp) / 2;
					
					f += fm;
					r += recall;
					p += precision;
					n += 1;
					PSNR += psnr;
					NRM += nrm;
					System.out.println("recall: " + recall);
					System.out.println("precision: " + precision);
					System.out.println("fm: " + fm);
					System.out.println("psnr: " + psnr);
					System.out.println("nrm: " + nrm);
					
				}
			
		
			System.out.println("\n\n=============FINAL================");
			
			System.out.println("Recal: "+ r / n);
			System.out.println("Precision: "+ p / n);
			System.out.println("f_mean: "+ f / n);
			System.out.println("psnr: " + PSNR / n);
			System.out.println("nrm: " + ((NRM / n) * Math.pow(10, -2)));
	
	}
	
	
	public static IBinaryImage getBinary(IGrayScaleImage img, int param, int k) throws IOException{
		
		UltimateOpening uo = new UltimateOpening();
		
		MaxtreeBuilder mb = new MaxtreeBuilder();
		mb.buildMaxtree(img.invert(), 1);
		uo.runUO(mb, param);
		IGrayScaleImage imgUAO = uo.getTransformImg();
		
		int indexT = new ThresholdGlobal(imgUAO).getIndexOtsu();
		if(indexT > 30)
			indexT = 30;
		
		IBinaryImage imgRb = new ThresholdLocal(imgUAO).otsuLocal(indexT);
		
		double std = getStd(imgRb);
		if(std > 30 && k > 0){
			return getBinary(img, (int)(param * .3), 0);
		}
		
		IBinaryImage imgRbArea = MorphologicalOperator.areaOpen(imgRb, 10);
		IBinaryImage imgOut = imgRbArea.duplicate();
		
		int wid = 5;
		int hei = 5;
		for(int w=0;w<imgRbArea.getWidth();w++)
			for(int h=0;h<imgRbArea.getHeight();h++){		
				if(!imgRb.isPixelBackground(w, h) && imgRbArea.isPixelBackground(w, h)){
					int cont = 0;
					int total = 0;
					double ratio;
					for(int i=0; i != wid;i++)
						for(int j = 0;j != hei;j++){
							int ww = w + i -wid/2;
							int hh = h + j - hei/2;
							if(ImageUtils.isIndexInImage(imgRbArea, ww, hh)){
								if(!imgRbArea.isPixelBackground(ww, hh)){
									cont++;
								}
								total++;
							}
						}
					ratio = (double) cont/total;
					if(ratio < 0.3)
						imgOut.setPixel(w, h, imgOut.getBackground());
				}
			}
		return imgOut;

	}
	
	public static double getStd(IBinaryImage imgRb){
		ArrayList<Integer> list = MorphologicalOperator.getHeights(imgRb);
		double avg = 0;
		for(Integer x: list)
			avg += x;
		avg = avg / list.size();
		
		double std = 0;
		for(Integer x: list){
			std += Math.pow(avg - x, 2);
		}
		std = Math.sqrt(std / list.size());
		return std;
	}

}
