package mmtext.images;

import java.awt.Image;
import java.awt.Point;

import mmtext.images.utils.ImageBuilder;




public class BinaryImage implements IBinaryImage {
    private int width; //largura da imagem
    private int height; //altura da imagem
    private int pixels[][]; //matriz de pixel da imagem
    private int background; //valor do pixel que representara o background da imagem
    private int foreground; //valor do pixel que representara o foreground da imagem
    public static final int COLOR_BINARY_IMAGE_BLACK = 0;
    public static final int COLOR_BINARY_IMAGE_WHITE = 255;
    public static final int DONT_CARE = -1;
    private Point origin; //origem da imagem
    private int id;
    
	

	/**
     * Construtor para criar uma nova imagem
     * @param width - largura
     * @param height - altura 
     */
    public BinaryImage(int width, int height) {
        this.width = width;
        this.height = height;
        this.foreground = COLOR_BINARY_IMAGE_BLACK;
        this.background = COLOR_BINARY_IMAGE_WHITE;
        pixels = new int[width][height];
    }

    public BinaryImage(int matrix[][]){
    	this.width = matrix.length;
    	this.height = matrix[0].length;
    	this.pixels = new int[width][height];
    	for (int i = 0; i < getWidth(); i++)
    		for (int j = 0; j < getHeight(); j++)
    			pixels[i][j] = (int) matrix[i][j];
    }
    
    /**
     * Inicializar todos os pixel da imagem para um dado nivel de cinza
     * @param color
     */
    public void initImage(int color){
        if(!(color == COLOR_BINARY_IMAGE_BLACK || color == COLOR_BINARY_IMAGE_WHITE || color == DONT_CARE)) throw new RuntimeException("Color ("+color+") invalid!");
        for (int i = 0; i < getWidth(); i++){
            for (int j = 0; j < getHeight(); j++){
                setPixel(i,j, color);
            }
        }
    }
    
    /**
     * Cria uma copia da imagem original
     * @return BinaryImage - nova imagem
     */
    public BinaryImage duplicate(){
        BinaryImage clone = new BinaryImage(getWidth(), getHeight());
        clone.background = background;
        clone.foreground = foreground;
        for (int i = 0; i < getWidth(); i++){
            for (int j = 0; j < getHeight(); j++){
                clone.setPixel(i,j,getPixel(i, j));
            }
        }
        return clone;
    }
    
    /**
     * Pega o valor do pixel (x, y)
     * @param x - largura
     * @param y - altura
     * @return float - valor do pixel
     */
    public int getPixel(int x, int y){
        if(x<0 || y < 0 || x >= getWidth() || y >= getHeight()) return 0;// throw new RuntimeException("(x="+x+", y="+y+") Nao existe essa posicao na matriz de pixels");
        return pixels[x][y];
    }
    
    /**
     * Pega o valor do pixel (x, y)
     * @param x - largura
     * @param y - altura
     * @return float - valor do pixel
     */
    public int getPixelBinary(int x, int y){
        if(x<0 || y < 0 || x >= getWidth() || y >= getHeight())return 0;// throw new RuntimeException("(x="+x+", y="+y+") Nao existe essa posicao na matriz de pixels");
        if(pixels[x][y] == getForeground()) return 1;
        else return 0;
    }
    
    
    /**
     * Modifica o valor do pixel (x, y) = value
     * @param x - largura
     * @param y - altura
     * @param value - valor do pixel
     */
    public void setPixel(int x, int y, int value){
        if(x >= getWidth() || y >= getHeight()) throw new RuntimeException("(x="+x+", y="+y+") Nao existe essa posicao na matriz de pixels");
      //  if(!(value == COLOR_BINARY_IMAGE_BLACK || value == COLOR_BINARY_IMAGE_WHITE || value == DONT_CARE)) throw new RuntimeException("Color ("+value+") invalid!");
        pixels[x][y] = (int) value;
    }
    
    /**
     * Pega uma matriz bidimensional de pixel da imagem
     * @return int[][]
     */
    public int[][] getPixels(){
    	int mat[][] = new int[width][height];
        for (int i = 0; i < getWidth(); i++)
            for (int j = 0; j < getHeight(); j++)
                mat[i][j] = pixels[i][j];
        return mat;
    }
    
    /**
     * Modifica a matriz de pixel da imagem para os valores da matriz dada
     * @param matrix 
     */
    public void setPixels(int matrix[][]){
        this.width = matrix.length;
        this.height = matrix[0].length;
        this.pixels = new int[width][height];
        for (int i = 0; i < getWidth(); i++){
            for (int j = 0; j < getHeight(); j++){
                setPixel(i, j, matrix[i][j] > 150? BinaryImage.COLOR_BINARY_IMAGE_WHITE : BinaryImage.COLOR_BINARY_IMAGE_BLACK);
            }
        }
    }
    
    public void setMatrix(int matrix[][]){
        this.width = matrix.length;
        this.height = matrix[0].length;
        this.pixels = new int[width][height];
        for (int i = 0; i < getWidth(); i++)
            for (int j = 0; j < getHeight(); j++)
                pixels[i][j] = (int) matrix[i][j];
    }
    
    /**
     * Retorna a altura da imagem
     * @return int - altura
     */
    public int getHeight(){
        return height;
    }
    
    /**
     * Retorna o tamanho da imagem
     * @return
     */
    public int getSize(){
        return getHeight() * getWidth();
    }
        
    /**
     * Retorna a largura da imagem
     * @return int - largura
     */
    public int getWidth(){
        return width;
    }
    
    
    /**
     * pega o valor do background das imagens binarias
     * @return
     */
    public int getBackground() {
        return background;
    }

    /**
     * Modifica a cor do background da imagem
     * @param background
     */
    public void setBackground(int background){
        if(background == COLOR_BINARY_IMAGE_BLACK){
            this.background = COLOR_BINARY_IMAGE_BLACK;
            this.foreground = COLOR_BINARY_IMAGE_WHITE;
        }else{
            this.background = COLOR_BINARY_IMAGE_WHITE;
            this.foreground = COLOR_BINARY_IMAGE_BLACK;
        }
    }

    /**
     * Pega o valor do foreground das imagens binarias
     * @return
     */
    public int getForeground() {
        return foreground;
    }


    /**
     * Pega a posicao de x do ponto de origem da imagem
     * @return Point 
     */
    public int getOriginX(){
        return (int) origin.getX();
    }
    
    /**
     * Pega a posicao de y do ponto de origem da imagem
     * @return Point 
     */
    public int getOriginY(){
        return (int) origin.getY();
    }
    
    /**
     * Modifica o valor da origem do elemento estruturante
     * @param x
     * @param y
     */
    public void setOrigin(int x, int y){
        origin = new Point(x, y);
    }
    
    /**
     * Converte uma imagem binaria em niveis de cinza
     * @return IGrayScaleImage
     */
    public GrayScaleImage convertGrayScale(){
        GrayScaleImage clone = new GrayScaleImage(getWidth(), getHeight());
        for (int i = 0; i < getWidth(); i++){
            for (int j = 0; j < getHeight(); j++){
                clone.setPixel(i,j,getPixel(i, j));
            }
        }
        return clone;
    }
    
    public int getArea(){
        int area =0;
        for(int w=0; w < getWidth(); w++){
            for(int h=0; h < getHeight(); h++){
                if(getPixel(w, h) == getForeground()){
                    area++;
                }
            }
        }
        return area;
    }

    
    
    /**
     * Verifica se duas imagens sao iguais
     * @param img - IGrayScaleImage
     * @return true se forem iguais false caso contrario
     */
    public boolean equals(Object o){
        BinaryImage img = (BinaryImage) o;
        for(int x = 0 ; x < getWidth() ; x++)
            for(int y = 0 ; y < getHeight(); y++)
                if(getPixel(x, y) != img.getPixel(x, y)) 
                    return false;
                
        return true;
    }

    /**
     * Inverte as tonalidades de cores da imagem
     */
    public BinaryImage invert() {
        BinaryImage clone = new BinaryImage(getWidth(), getHeight());
        clone.background = foreground;
        clone.foreground = background;
        int value;
        for (int i = 0; i < getWidth(); i++){
            for (int j = 0; j < getHeight(); j++){
                if(getPixel(i, j) == foreground)
                    value = background;
                else if (getPixel(i, j) == background)
                    value = foreground;
                else
                    value = getPixel(i, j);
                clone.setPixel(i,j,value);
            }
        }
        return clone;
    }
        
    /**
     * Pega o histograma da projecao dos pixels em X
     */
    public int[] getHistogramXprojection() {
        int hist[] = new int[this.getWidth()];
        for(int i=0; i < this.getWidth(); i++){
            for(int j=0; j < this.getHeight(); j++){
                if(this.getPixel(i, j) == this.getForeground()){
                    hist[i]++;
                }
            }
        }
        return hist;
    }

    /**
     * Pega o histograma da projecao dos pixels em Y
     */
    public int[] getHistogramYprojection() {
        int hist[] = new int[this.getHeight()];
        for(int i=0; i < this.getWidth(); i++){
            for(int j=0; j < this.getHeight(); j++){
                if(this.getPixel(i, j) == this.getForeground()){
                    hist[j]++;
                }
            }
        }
        return hist;  
    }
    

    public void setPixel(int i, int level){
        int w = i / getHeight();
        int h = i % getHeight();
        pixels[w][h] = (int) level;
    }
    
    public int getPixel(int i){
        int w = i / getHeight();
        int h = i % getHeight();
        return pixels[w][h];
    }
    
    public boolean isPixelBackground(int i){
        return getPixel(i) == getBackground();
    }
    
    public boolean isPixelBackground(int x, int y){
        return getPixel(x, y) == getBackground();
    }

	public Image createImage() {
		int p[][] = new int[width][height];
		for(int w=0; w < width; w++)
			for(int h=0; h < height; h++)
				p[w][h] = pixels[w][h];
		
		return ImageBuilder.convertToImage(p);
	}
   

    /** 
     * Faz a reflexao de uma imagem
     * @param img - imagem de entrada
     * @return BinaryImage com a reflexao da imagem de entrada
     */
    public IBinaryImage reflection(IBinaryImage img){
        IBinaryImage imgOut = new BinaryImage(img.getWidth(), img.getHeight());
        imgOut.setBackground(img.getBackground());
        for(int x = 0 ; x < img.getWidth(); x++)
            for(int y = 0 ; y < img.getHeight(); y++)
                imgOut.setPixel(img.getWidth() - x -1, img.getHeight()- y -1, img.getPixel(x,y));
        return imgOut;
    }
    
    public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}
    

    /**
     * Pegando um boundbox
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @return
     */
    public BoundBox getBoundBox(int x1, int y1, int x2, int y2){
        BoundBox out = new BoundBox(){
            private int x1, y1, x2, y2;
            
            public int getPixelForeground(){
                return foreground;
            }
            
            public void setPixelForeground(int c){
                if(c == COLOR_BINARY_IMAGE_BLACK){
                    setBackground(COLOR_BINARY_IMAGE_WHITE);
                }else{
                    setBackground(COLOR_BINARY_IMAGE_BLACK);
                }
            }
            
            public Point getPointBegin() {
                return new Point(x1, y1);
            }

            public Point getPointEnd() {
                return new Point(x2, y2);
            }
            
            public void setDimension(int x1, int y1, int x2, int y2){
                //if(x1 >= getWidth() || y1 >= getHeight() || x2 >= getWidth() || y2 >= getHeight()) throw new RuntimeException("Tamanho do boundbox invalido!");
                this.x1 = x1;
                this.x2 = x2;
                this.y1 = y1;
                this.y2 = y2;
            }
            
            public void setDimension(int x1, int y1, int x2, int y2, int fore){
                //if(x1 >= getWidth() || y1 >= getHeight() || x2 >= getWidth() || y2 >= getHeight()) throw new RuntimeException("Tamanho do boundbox invalido!");
                this.x1 = x1;
                this.x2 = x2;
                this.y1 = y1;
                this.y2 = y2;
            }
            
            public int getPixel(int w, int h) {
                w += x1;
                h += y1;
                return pixels[w][h];
            }

            public void setPixel(int w, int h, int value) {
                
                // if(w < x1 || w > x2 || h < y1 || h > y2) throw new RuntimeException("(x="+w+", y="+h+") Nao existe essa posicao no boundbox");
                w += x1;
                h += y1;
                pixels[w][h] = value;
            }

            public void paintBoundBox(int color){
                int w = x2 - x1;
                int h = y2 - y1;
                for(int i=0; i < w; i++){
                    for(int j=0; j < h; j++){
                        if(i == 0 || j == 0 || i == w-1 || j == h-1)
                            pixels[x1+i][y1+j] = color;      
                    }
                }
            }
            
            public IBinaryImage getImage() {
                IBinaryImage out = new BinaryImage(x2 - x1, y2 - y1);
                out.setBackground(background);
                for(int i=0; i < out.getWidth(); i++){
                    for(int j=0; j < out.getHeight(); j++){
                        out.setPixel(i, j, getPixel(i, j));
                    }
                }
                return out;
            }

            public int getWidth() {
                return x2 - x1 + 1;
            }

            public int getHeight() {
                return y2 - y1 + 1;
            }

            public int countPixels(int color) {
                int area =0;
                for(int w=0; w < getWidth(); w++){
                    for(int h=0; h < getHeight(); h++){
                        if(getPixel(w, h) == color){
                            area++;
                        }
                    }
                }
                return area;
            }

            public void initPixels(int color) {
                for(int i=0; i < getWidth(); i++){
                    for(int j=0; j < getHeight(); j++)
                        setPixel(i, j, color);
                }
            }
            
        };
        out.setDimension(x1, y1, x2, y2, -1);
        return out;
    }
}
