package mmtext.images;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;

import mmtext.images.descriptor.Moment;
import mmtext.images.utils.ImageGraphics;
import mmtext.images.utils.ImageUtils;
import mmtext.images.utils.Resampling;

public class MinimalBoundBox implements Comparable<MinimalBoundBox> {

    private Point minH;
    private Point minW;
    private Point maxH;
    private Point maxW;
    private double angleInRadian;
    private double angleInDegrees;
    private IBinaryImage img;
    private Point centroid;
    private double distance;
    private int label;
    //private IBinaryImage imgRot;
    private String xml;
    private double angleInRadianImgRot;
    
    public String getXML(){
    	return xml;
    }
    
    
    /**
     * Rotacao em sentido anti-horario
     * @param img - imagem de entrada
     * @param graus - graus
     * @return imgem de saida
     */
    public int getPixelRotation(int w, int h){
         double radian = 2.0 * Math.PI * angleInDegrees / 360.0;
        double ct = Math.cos(radian);
        double st = Math.sin(radian);
        
        int hc = img.getHeight()/2;
        int wc = img.getWidth()/2;
        
        int nh = (int) ((h - hc) * ct - (w - wc) * st + hc);
        int nw = (int) ((h - hc) * st + (w - wc) * ct + wc);
        if(ImageUtils.isIndexInImage(img, nw, nh))
        	return img.getPixel(nw, nh);
        else
        	return img.getBackground();
       
    }
    
    public void calculateDistance(int x, int y){
    	distance = ImageUtils.distance(x, y, centroid.getX(), centroid.getY(), 1);
    }
    
    public MinimalBoundBox(IBinaryImage img, boolean b, int label){
    	this.img = img;
    	this.label = label;
    	this.centroid = Moment.getCentroid(img);
    	calculateDistance(0,0);
    	if(b){
    		this.angleInRadian = Moment.getOrientationEstimationInRadian(img);
            this.angleInDegrees = Moment.toDegreesForRegionText(angleInRadian);
    	}else{
    		this.angleInRadian = -1;
            this.angleInDegrees = -1;
    	}
    	
    	//this.imgRot = Resampling.rotation(img, angleInDegrees);
    	//this.angleInRadianImgRot = Moment.getOrientationEstimationInRadian(this.imgRot);

    	int minHeight = Integer.MAX_VALUE;
    	int minWidth = Integer.MAX_VALUE;
    	int maxHeight = Integer.MIN_VALUE;
    	int maxWidth = Integer.MIN_VALUE;
    	int fore = img.getForeground();
    	for(int i=0; i < img.getWidth(); i++){
    		for(int j=0; j < img.getHeight(); j++){
    			if( (!b && !img.isPixelBackground(i, j))  || (b && getPixelRotation(i, j) == fore)){
    				if(minHeight > j){
    					minHeight = j;
    					minH = new Point(i, j);
    				}
    				if(minWidth > i){
    					minWidth = i;
    					minW = new Point(i, j);   
    				}
    				if(maxHeight < j){
    					maxHeight = j;
    					maxH = new Point(i, j);   
    				}
    				if(maxWidth < i){
    					maxWidth = i;
    					maxW = new Point(i, j);
    				}
    			}
    		}
    	}	
    	
    }
    
    public MinimalBoundBox(MinimalBoundBox box, IBinaryImage img){
    	this.minH = box.minH;
    	this.minW = box.minW;
    	this.maxH = box.maxH;
    	this.maxW = box.maxW;
    	this.angleInRadian = box.angleInRadian;
    	this.angleInDegrees = box.angleInDegrees;
    	this.img = img;
    	this.centroid = box.centroid;
    	this.distance = box.distance;
    	this.label = box.label;
    	this.angleInRadianImgRot = box.angleInRadianImgRot;
    	
    	
    	int minWidth = Integer.MAX_VALUE;
    	int maxWidth = Integer.MIN_VALUE;
    	int fore = img.getForeground();
    	for(int i=0; i < img.getWidth(); i++){
    		for(int j=0; j < img.getHeight(); j++){
    			if( (!img.isPixelBackground(i, j))  || (getPixelRotation(i, j) == fore)){
    				if(minWidth > i){
    					minWidth = i;
    					minW = new Point(i, j);   
    				}
    				if(maxWidth < i){
    					maxWidth = i;
    					maxW = new Point(i, j);
    				}
    			}
    		}
    	}
    	
    }
    
    public MinimalBoundBox(IBinaryImage img){
        this(img, true, -1);
    }
    
    public Rectangle getRectangle(){
    	return new Rectangle(minW.x, minH.y, maxW.x - minW.x, maxH.y -  minH.y);
    }
    
    public int getLabel(){
    	return label;
    }
    
    public boolean isMergeByCriterio1(MinimalBoundBox box){
        //criterio a: angulos inferior a 15 graus
        
        double min = Double.MAX_VALUE;
        double theta1 = this.getAngleInDegrees();
        double theta2 = box.getAngleInDegrees();
        double n1 = Math.abs(theta1 - theta2);
        double n2 = Math.abs(theta1 - theta2 + 360);
        double n3 = Math.abs(theta1 - theta2 - 360);
        min = n1;
        if(min > n2)
            min = n2;
        if(min > n3)
            min = n3;
        

        //criterio b: Alturas semelhantes
        double hR1 = this.getHeight();
        double hR2 = box.getHeight();
        
        n1 = Math.abs(hR1 - hR2);
        n2 = Math.min(hR1, hR2);

        return (n1 < n2) && (min < 15);

    }
    
    public boolean isMergeByCriterio2(MinimalBoundBox box){
        //Distancia entre os centroides
        Point cent1 =  Moment.getCentroid(this.getImage());
        Point cent2 =  Moment.getCentroid(box.getImage());
       
        double dx = ImageUtils.distance(cent1.getX(), cent1.getY(), cent2.getX(), cent2.getY(), 1);
        
        boolean result1 = dx < Math.max(this.getWidth(),  box.getWidth());
        
        
       //Distancia entre as retas principais
    //   Point centR1 = this.getImageRotation().getCentroid();
    //   Point centR2 = box.getImageRotation().getCentroid(); 
      
       Point centR1 = cent1;// this.getImage().getCentroid();
       Point centR2 = cent2;//box.getImage().getCentroid(); 
          
        
       double mr1 = Math.abs(Math.tan( this.getAngleInRadianImgRot() ));
       double mr2 = Math.abs(Math.tan( box.getAngleInRadianImgRot() ));
       
       double br1 = centR1.getY() - centR1.getX() * mr1;
       double br2 = centR2.getY() - centR2.getX() * mr2;
       
       double d = Math.abs(centR2.getY() - mr1 * centR2.getX() - br1) / (2 * Math.sqrt(1 + Math.pow(mr1, 2) )) + Math.abs(centR1.getY() - mr2 * centR1.getX() - br2) / (2 * Math.sqrt(1 + Math.pow(mr2, 2)));
       
       boolean result2 = d < Math.min(this.getHeight(), box.getHeight());

       return result1 && result2;
    }
    
    public MinimalBoundBox invert(){
        MinimalBoundBox box = new MinimalBoundBox(this.getImage().invert());
        box.maxH = this.maxH;
        box.maxW = this.maxW;
        box.minH = this.minH;
        box.minW = this.minW;
        return box;
    }
    
    public IBinaryImage getImage(){
        return img;
    }
    
  //  public IBinaryImage getImageRotation(){
  //      return imgRot;
  //  }
    
    public int getWidth(){
        return  maxW.x - minW.x; 
    }
    public int getHeight(){
        return maxH.y -  minH.y;
    }
    
    public double getAngleInDegrees() {
        return angleInDegrees;
    }

    public double getAngleInRadianImgRot() {
        return angleInRadianImgRot;
    }


    public double getAngleInRadian() {
        return angleInRadian;
    }

    public Point getMinHeightPoint(){
        return minH;
    }
    
    public Point getMaxHeightPoint(){
        return maxH;
    }
    
    public Point getMinWidthPoint(){
        return minW;
    }
    
    public Point getMaxWidthPoint(){
        return maxW;
    }
    
    public void paint(IRGBImage img, Color c){
    	paint(img, c, 1);
    }
    
    public void paint(IRGBImage img, Color c, float fator){
    
    	if(this.angleInDegrees == -1){
    		ImageGraphics.drawRectangle(img, new Point(minW.x, minH.y), new Point(maxW.x, maxH.y), c);
    		return;
    	}
    	
    	//   ImageGraphics.lineWidth = 1;
    	double angle = this.angleInDegrees;
    	IBinaryImage imgRot = Resampling.rotationBinary(this.img, angle);
    	IRGBImage imgTmp = new RGBImage(imgRot);
    	
        ImageGraphics.drawRectangle(imgTmp, new Point(minW.x, minH.y), new Point(maxW.x, maxH.y), c);
        
        xml  = "<taggedRectangle x='"+(int)(minW.x / fator)+"' y='"+ (int)(minH.y / fator) +"' width='"+(int)((maxW.x / fator)- (minW.x / fator) )+"' height='"+(int)((maxH.y / fator) - (minH.y / fator))+"'/>\n";
        
        double radian = 2.0 * Math.PI * -angle / 360.0;
        double ct = Math.cos(radian);
        double st = Math.sin(radian);
        
        int hc = img.getHeight()/2;
        int wc = img.getWidth()/2;
        int nh, nw;
        
        
        
        for(int w=0; w < img.getWidth(); w++){ 
            for(int h=0; h < img.getHeight(); h++){
                nh = (int) ((h - hc) * ct - (w - wc) * st + hc);
                nw = (int) ((h - hc) * st + (w - wc) * ct + wc);
                if(ImageUtils.isIndexInImage(img, nw, nh) &&ImageUtils.isIndexInImage(imgTmp, nw, nh) &&imgTmp.getPixel(nw, nh).equals(c))
                    img.setPixel(w, h, c);
            }
        }
    }
    
    
    
    /*
    public static MinimalBoundBox getCompare(int label){
    	return new MinimalBoundBox(label);
    }

	@Override
	public int compareTo(MinimalBoundBox o) {
		int l = o.label;
		if(label > l)
			return 1;
		else if(l > label)
			return -1;
		else
			return 0;
	}
	*/
    
    public Point getCentroid(){
    	return centroid;
    }
    
	public int compareTo(MinimalBoundBox o) {
		return (int) (distance - o.distance);
	}
    
}
