package mmtext.images.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;

import mmtext.images.BinaryImage;
import mmtext.images.BoundBox;
import mmtext.images.IBinaryImage;
import mmtext.images.IRGBImage;
import mmtext.images.RGBImage;
import mmtext.images.descriptor.Moment;
import mmtext.images.utils.ImageGraphics;
import mmtext.images.utils.ImageUtils;
import mmtext.images.utils.LinearRegression;

/**
 * 
 * @author wonder
 * Classe utilizado para exibir o histograma da projecao x da imagem na tela
 */
public class WindowXProjection {
    
    private final int HIST_WIDTH;
    private int YMARGIN = 10;
    private final int XMARGIN = 20;
    private final int WIN_WIDTH ;
    private final int WIN_HEIGHT;
    private final int THRESHOLD_THICKNESS_CHAR;
    private final int HIST_HEIGHT;
    private final int THRESHOLD;
    
    private IBinaryImage img;
    private int meanPeak;
    private int maxPeak;
    private int minPeak;
    private int[] hist;
    private int countCharacters = 0;
    private IBinaryImage imgCharacters[];
    private IRGBImage imgOut;
    private IRGBImage imgProjection;
    private int charWidthWithoutSpace = 0;
    private int charHeightWithoutSpace = 0;
    private int charWidth = 0;
    private int charHeight = 0;
    private double charWidthVariance = 0;
    private double charHeightVariance = 0;
    private double linearity;
    private double coffRegression;
    private ArrayList<BoundBox> boundBoxCharacters = new ArrayList<BoundBox>();;
    private boolean isProcessedGraphics = false;
    private LinearRegression regression;
    
    ArrayList<Integer> medianSpace = new ArrayList<Integer>();
    
    public WindowXProjection(IBinaryImage img, int threshold, int histHeight, int THRESHOLD_THICKNESS_CHAR){
        this.img = img;
        this.HIST_HEIGHT = histHeight;
        this.HIST_WIDTH = img.getWidth() ;
        if(img.getWidth() < 305)
            this.WIN_WIDTH = 305 + 2 * XMARGIN;
        else
            this.WIN_WIDTH = img.getWidth() + 2 * XMARGIN;
        this.WIN_HEIGHT = HIST_HEIGHT + img.getHeight() +  11 *YMARGIN;
        this.THRESHOLD = threshold;
        this.THRESHOLD_THICKNESS_CHAR = THRESHOLD_THICKNESS_CHAR;
        run();
    }
  
    public WindowXProjection(IBinaryImage img, int threshold, int histHeight){
        this(img, threshold, histHeight, 8);
    }
    
    public WindowXProjection(IBinaryImage img, int threshold){
        this(img, threshold, 100, 8);
    }
    
    public WindowXProjection(IBinaryImage img){
        this(img, img.getHeight() / 8, 100, 2);
    }
    
    public IBinaryImage[] getCharacters(){
        return this.imgCharacters;
    }
    
    public int getThreshold() {
        return THRESHOLD;
    }

    public IBinaryImage getImage(){
    	return img;
    }

    public int getCountCharacters() {
        return countCharacters;
    }

    public int[] getHistogram() {
        return hist;
    }

    public int getHIST_HEIGHT() {
        return HIST_HEIGHT;
    }

    public int getMaxPeak() {
        return maxPeak;
    }

    public int getMeanPeak() {
        return meanPeak;
    }

    public int getMinPeak() {
        return minPeak;
    }

    public int getCharacterWidthWithoutSpace(){
        return charWidthWithoutSpace;
    }
    
    public int getCharacterHeightWithoutSpace(){
        return charHeightWithoutSpace;
    }
    
    public int getCharacterWidth(){
        return charWidth;
    }
    
    public int getCharacterSpaceWidth(){
    	int x = getCharacterWidth() - getCharacterWidthWithoutSpace();
    	if(x < 3) x = 3;
    	return x;
    }
    
    public int getCharacterMedianWidth(){
    	if(medianSpace.size() > 0){
    		Collections.sort(medianSpace);
    		return medianSpace.get(medianSpace.size() / 2);
    	}
    	return 0;
    }
    
    public int getCharacterHeight(){
        return charHeight;
    }
    
    public double getCharacterWidthVariance(){
        return charWidthVariance;
    }
    
    public double getCharacterHeightVariance(){
        return charHeightVariance;
    }
    
    public double getCoffRegression(){
        return coffRegression;
    }
    
    /**
     * Desenha o histograma
     * @param img
     * @return
     */
    public IRGBImage getProjection(){
        if(!isProcessedGraphics){
           // ImageGraphics.drawRectangle(imgOut, new Point(XMARGIN-1, YMARGIN-1), new Point(img.getWidth() + XMARGIN, img.getHeight() + YMARGIN), Color.BLUE);
          //  try{    
          //      ImageGraphics.drawLine(imgOut, new Point(regression.getPointInitial().x + XMARGIN, regression.getPointInitial().y + YMARGIN), new Point(regression.getPointEnd().x + XMARGIN, regression.getPointEnd().y + YMARGIN), Color.RED);
          //  }catch(Exception e){}
            
            int limiar = (int)((double) this.HIST_HEIGHT * this.THRESHOLD) / maxPeak;
            
            //Desenhando os linhas do limiar e do pico medio
            int meanPeak2 = (int)((double)HIST_HEIGHT*meanPeak)/maxPeak;
            ImageGraphics.drawLine(imgOut, new Point(XMARGIN, YMARGIN + HIST_HEIGHT - meanPeak2), new Point(HIST_WIDTH+XMARGIN-1, YMARGIN + HIST_HEIGHT - meanPeak2), Color.RED);
            ImageGraphics.drawLine(imgOut, new Point(XMARGIN, YMARGIN + HIST_HEIGHT - limiar), new Point(HIST_WIDTH+XMARGIN-1, YMARGIN + HIST_HEIGHT -limiar), Color.MAGENTA);
            ///
            
            int x = XMARGIN + 1;
            int y = YMARGIN + HIST_HEIGHT + 5;
            drawText(x, y);
        }
        isProcessedGraphics = true;

        return imgOut;
    }
    
    /**
     * Desenha o histograma
     * @param img
     * @return
     */
    public void run(){
        imgOut = new RGBImage(WIN_WIDTH, WIN_HEIGHT);
        imgOut.initImage(255);
        regression = new LinearRegression(img);
        this.coffRegression = regression.r2();
        //calculando o histograma
        int sum=0;
        this.hist = new int[img.getWidth()];
        for(int i=0; i < img.getWidth(); i++){
            for(int j=0; j < img.getHeight(); j++){
                imgOut.setPixel(i + XMARGIN, j + YMARGIN, new Color(img.getPixel(i, j), img.getPixel(i, j), img.getPixel(i, j)));
                if(img.getPixel(i, j) == img.getForeground()){
                    hist[i]++;
                    sum++;
                }
            }
        }

        
        this.maxPeak = 0;
        this.minPeak = Integer.MAX_VALUE;
        for (int i = 0; i<hist.length; i++){
            if (hist[i] > maxPeak) {
                maxPeak = hist[i];
            }
            if(hist[i] != 0 && hist[i] < minPeak){
                minPeak = hist[i];
            }
        }
        meanPeak = sum / hist.length;
        YMARGIN += img.getHeight() + 5; 
        drawPlot();
        
        //contar caracteres
        int limiar = (int)((double) this.HIST_HEIGHT * this.THRESHOLD) / maxPeak;
        int countCut = -1;
        boolean flagCut1 = true;
        boolean flagCut2 = false;
        int initX = -1;
        ArrayList<IBinaryImage> listCharact = new ArrayList<IBinaryImage>();
        for(int i=XMARGIN; i < HIST_WIDTH+XMARGIN-1; i++){
            
            if(imgOut.getPixel(i, YMARGIN + HIST_HEIGHT - limiar,1) == 0){
                if(flagCut1 && countCut == 0){
                    //System.out.println("InicioX = "+ i);
                    initX = i;
                    flagCut1 = false;
                    countCharacters++;
                    ImageGraphics.drawLine(imgOut, new Point(i, YMARGIN+1), new Point(i, YMARGIN + HIST_HEIGHT-limiar), Color.ORANGE);
                }
                countCut++;
            }else{
                flagCut1 = true;
            }
            
            if(imgOut.getPixel(i, YMARGIN + HIST_HEIGHT - limiar,1) == 255){
                if(countCut > THRESHOLD_THICKNESS_CHAR && flagCut2 && flagCut1){
                    //System.out.println("FimX = "+ i +"\n\n");
                    flagCut2 = false;
                    countCut=0;
                    this.charWidthWithoutSpace += i - initX;
                    medianSpace.add(i - initX);
                    IBinaryImage imgC = new BinaryImage(i - initX, img.getHeight());
                    
                    for(int w=0; w < imgC.getWidth(); w++){
                        for(int h=0; h < imgC.getHeight(); h++){
                            int x = w + initX - XMARGIN;
                            imgC.setPixel(w, h, img.getPixel(x,h));
                        }
                    }
                    boundBoxCharacters.add(img.getBoundBox(initX - XMARGIN, 0, i - XMARGIN, img.getHeight()));
                    listCharact.add(imgC);
                    initX = -1;
                    ImageGraphics.drawLine(this.imgOut, new Point(i-1, YMARGIN+1), new Point(i-1, YMARGIN + HIST_HEIGHT-limiar), Color.GREEN);
                }
            }else{
                flagCut2 = true;
            }

        }
        if(initX != -1){
            IBinaryImage imgC = new BinaryImage(HIST_WIDTH+XMARGIN-1 - initX, img.getHeight());
            this.charWidthWithoutSpace += HIST_WIDTH+XMARGIN-1 - initX;
            medianSpace.add(HIST_WIDTH+XMARGIN-1 - initX);
            for(int w=0; w < imgC.getWidth(); w++){
                for(int h=0; h < imgC.getHeight(); h++){
                    int x = w + initX -XMARGIN;
                    imgC.setPixel(w, h, img.getPixel(x,h));
                }
            }
            listCharact.add(imgC);
        }
        
        if(listCharact.size() == 0) return;
        
        this.charWidthWithoutSpace = this.charWidthWithoutSpace / listCharact.size();
        this.charHeight = img.getHeight();
        this.charWidth = img.getWidth() / listCharact.size();
        this.imgCharacters = new IBinaryImage[listCharact.size()];
        listCharact.toArray(this.imgCharacters);
        //WindowImages.show(this.imgCharacters, false);
        int vHeight[] = new int[listCharact.size()];
        
        this.charHeightWithoutSpace = 0;
        for(int i=0; i < vHeight.length; i++){
            vHeight[i] = getHeightChar(imgCharacters[i]);
            this.charHeightWithoutSpace += vHeight[i];
        }
        this.charHeightWithoutSpace = this.charHeightWithoutSpace / listCharact.size();
        
        //calculando a variancia
        for(int i=0; i < imgCharacters.length; i++){
            this.charWidthVariance += Math.pow(imgCharacters[i].getWidth() - charWidthWithoutSpace, 2);
            this.charHeightVariance += Math.pow(vHeight[i] - charHeightWithoutSpace, 2);
        }
        this.charWidthVariance /= imgCharacters.length;
        this.charHeightVariance /= imgCharacters.length;
        
        
        double m = regression.getBeta1();
        double b = regression.getBeta0();
        this.linearity = 0;
        for(int i=0; i < imgCharacters.length; i++){
            Point p = Moment.getCentroid(imgCharacters[i]);
            //double mCi = Math.abs(Math.tan( Moment.getOrientationEstimationInRadian(imgCharacters[i]) ));            
            //double bCi = p.y - p.x * mCi;
            this.linearity += Math.abs((p.y - m * p.x + b) / (Math.pow(m, 2)-1));
        }
        this.linearity /= imgCharacters.length;        

        
    }
    
    
    private int getHeightChar(IBinaryImage imgC){
        return ImageUtils.adjustImage(imgC).getHeight();
    }
    
    /**
     * Desenha o texto no histograma
     * @param imgOut
     * @param histogram
     * @param x
     * @param y
     */
    private void drawText(int x, int y) {
        NumberFormat formatter = new DecimalFormat("0.000");
        ImageGraphics.drawString(imgOut, "Peak Min: " + minPeak, new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Characters: " + countCharacters, new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Peak Mean: " + meanPeak, new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Img Width: " + img.getWidth(), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Peak Max: " + maxPeak, new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Img Height: " + img.getHeight(), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Char. height: " + charHeight, new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Char. Width : " + charWidth, new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Char. height (without space): " + charHeightWithoutSpace, new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Char. Width (without space): " + charWidthWithoutSpace, new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Variance Height: " +formatter.format(charHeightVariance), new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Variance Width: " + formatter.format(charWidthVariance), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
        y += 12;
        ImageGraphics.drawString(imgOut, "Linearity: " +formatter.format(linearity), new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Regression coefficient: " + formatter.format(coffRegression), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 170, y);
    }
    
    public double getLinearity(){
        return linearity;
    }
    
    public ArrayList<BoundBox> getBoundBoxCharacters(){
        return boundBoxCharacters;
    }
    
    /**
     * Desenhar os picos do histograma
     * @param img
     * @param maxCount
     * @param h
     */
    private void drawPlot() {
        this.imgProjection = new RGBImage(img.getWidth(), img.getHeight());
        this.imgProjection.initImage(255);
        int maxCount = maxPeak;
        if (maxCount==0) maxCount = 1;
        ImageGraphics.drawRectangle(imgOut, new Point(XMARGIN, YMARGIN), HIST_WIDTH, HIST_HEIGHT+2, Color.BLUE);
        int  y;
        for (int i = 0; i < hist.length; i++) {
            y = (int)((double)HIST_HEIGHT*hist[i])/maxCount;
            if (y>HIST_HEIGHT)
                y = HIST_HEIGHT;
            ImageGraphics.drawLine(imgOut, new Point(i+XMARGIN, YMARGIN+HIST_HEIGHT), new Point(i+XMARGIN, YMARGIN+HIST_HEIGHT-y), Color.BLACK);
            ImageGraphics.drawLine(this.imgProjection, new Point(i, this.imgProjection.getHeight()-1), new Point(i, this.imgProjection.getHeight() - hist[i] -1), Color.BLACK);
        }
    }
    
    public void show(){
        WindowImages.show(this.getProjection(), "Localizao de textos em imagens", "X-Projeo");
    }
    
    public static WindowXProjection show(IBinaryImage img){
        WindowXProjection win = new WindowXProjection(img, img.getHeight() / 10, 100, 5);
        win.show();
        return win;
    }
    
    /**
     * Retorna a imagem do histograma da projecao
     * @return
     */
    public IRGBImage getHistogramProjection(){
        return this.imgProjection;
    }
  

}
