package mmtext.images.operators.uao;
import java.util.LinkedList;
import java.util.Queue;

import mmtext.images.GrayScaleImage;
import mmtext.images.IGrayScaleImage;

/**
 * Esta classe acessa a Maxtree e, atrav�s dela e das estruturas auxiliares l� alocadas,
 * realiza a opera��o de Attribute Opening na imagem
 * 
 * @author alexandremorimitsu
 *
 */
public class AttributeOpening {
	private IGrayScaleImage aoImage;
	private int[] aoLevel;
	private Queue<Integer> q;
	private MaxtreeBuilder mb;
	
	private Maxtree m;
	private double[] heights;
	
	void init(MaxtreeBuilder mBuilder){
		this.mb = mBuilder;
		aoImage = new GrayScaleImage(mb.getLabeledImg().getWidth(), mb.getLabeledImg().getHeight());
		aoLevel = new int[mb.getAttributeValues().length];
		q = new LinkedList<Integer>();
		m = mb.getM();
		heights = mb.getAttributeValues();
	}
	
	public void printaoLevel(){
		for(int i=0;i<aoLevel.length;i++)
			System.out.printf("ao[%d] = %d\n", i, aoLevel[i]);
	}
	
	public void printaoImage(){
		for(int h=0;h<aoImage.getHeight();h++){
			for(int w=0;w<aoImage.getWidth();w++)
				System.out.printf("%3d ", aoImage.getPixel(w, h));
			System.out.println();
		}
	}
	
	/*
	 * Faz uma busca em largura na Maxtree para atualizar a soma dos atributos
	 * (Computa��o do Attribute Opening)
	 * 
	 * @param totalSum
	 */
	public void computeaoLevel(int threshold){
		Node node;
		int parentId;
		int currentId;
		
		while(!q.isEmpty()){
			parentId = q.poll();
			node = m.getNode(parentId);
			currentId = node.getSonId();
			while(currentId != -1){
				q.add(currentId);
				if(heights[currentId] < threshold)
					aoLevel[currentId] = aoLevel[parentId];
				else
					aoLevel[currentId] = mb.getLevel().get(currentId);
				node = m.getNode(currentId);
				currentId = node.getBrotherId();
			}
		}
	}
	
	public void generateaoImage(){
		for(int w=0;w<aoImage.getWidth();w++)
			for(int h=0;h<aoImage.getHeight();h++){
				aoImage.setPixel(w, h, aoLevel[mb.getLabeledImg().getPixel(w, h)]);
			}		
	}

	public void runAttributeOpening(MaxtreeBuilder mb, int threshold){
		init(mb);
		q.add(0);
		aoLevel[0] = 0;
		computeaoLevel(threshold);
		generateaoImage();
	}
	
	public IGrayScaleImage run(IGrayScaleImage img, int typeParam, int paramValue){
		MaxtreeBuilder mb = new MaxtreeBuilder();
		mb.buildMaxtree(img, typeParam);
		
		init(mb);
		q.add(0);
		aoLevel[0] = 0;
		computeaoLevel(paramValue);
		generateaoImage();
	
		return getAoImage();
	}
	
	

	public int[] getAoLevel() {
		return aoLevel;
	}

	public void setAoLevel(int[] aoLevel) {
		this.aoLevel = aoLevel;
	}

	public Queue<Integer> getQ() {
		return q;
	}

	public void setQ(Queue<Integer> q) {
		this.q = q;
	}

	public MaxtreeBuilder getMb() {
		return mb;
	}

	public void setMb(MaxtreeBuilder mb) {
		this.mb = mb;
	}

	public Maxtree getM() {
		return m;
	}

	public void setM(Maxtree m) {
		this.m = m;
	}

	public IGrayScaleImage getAoImage() {
		return aoImage;
	}

	public void setAoImage(IGrayScaleImage aoImage) {
		this.aoImage = aoImage;
	}

	

	
	
}
