package mmtext.images.operators.uao;
import java.util.ArrayList;

public class Maxtree {
	private ArrayList<Page> maxtree;
	private int nodesPerPage;
	int numPages;
	
	public Maxtree(){
		maxtree = new ArrayList<Page>();
	}
	
	public void init(int imageSize){
		this.nodesPerPage = (int)Math.sqrt(imageSize/Math.log10(imageSize));
		/*a linha acima é uma gambiarra para tentar aproximar numero de Pages 
		 * com nodesPerPage ao final da execução do flood*/
		this.numPages = 0;
	}
	
	public void addPage(){//adiciona uma nova página à Maxtree
		maxtree.add(new Page(nodesPerPage));
		numPages++;
	}
	
	public void setNode(Node node, int nodeId){//seta o valor do nó nodeId para node
		if(nodeId/nodesPerPage >= maxtree.size())
			addPage();
		maxtree.get(nodeId/nodesPerPage).getPage()[nodeId%nodesPerPage] = node;
	}
	
	public Node getNode(int nodeId){
		return maxtree.get(nodeId/nodesPerPage).getPage()[nodeId%nodesPerPage];
	}
	
	public void printMaxtree(int node){
		for(int i=0;i<node;i++){
			int numPage = i/nodesPerPage;
			int numNode = i%nodesPerPage;
			int sId =  maxtree.get(numPage).getPage()[numNode].getSonId();
			int bId =  maxtree.get(numPage).getPage()[numNode].getBrotherId();
			System.out.printf("%3d: SId: %d\t, Bid: %d\n", i, sId, bId);
		}
	}
	
	//Getters and setters
	
	public ArrayList<Page> getMaxtree() {
		return maxtree;
	}

	public void setMaxtree(ArrayList<Page> maxtree) {
		this.maxtree = maxtree;
	}

	public int getNodesPerPage() {
		return nodesPerPage;
	}

	public void setNodesPerPage(int nodesPerPage) {
		this.nodesPerPage = nodesPerPage;
	}

	public int getNumPages() {
		return numPages;
	}

	public void setNumPages(int numPages) {
		this.numPages = numPages;
	}
}
