package mmtext.images.utils;

import java.awt.Color;
import java.awt.Point;

import mmtext.images.IBinaryImage;
import mmtext.images.IRGBImage;
import mmtext.images.RGBImage;

/**
 * Project: Computer Vision Framework
 * 
 * @author Wonder Alexandre Luz Alves
 * @date 30/01/2009
 * Metodo de regressao por minimos quadrados. Seu objetivo eh determinar os valores de a e b para funcao
 * y = ax+b de forma que y(x) seja a reta com melhor ajustamento de um conjunto de n pontos.
 */
public class LinearRegression { 
    private IBinaryImage img;
    private double beta1 = 0; //a
    private double beta0 = 0; //b
    private double r2 = 0;
    
    
    public LinearRegression(IBinaryImage img){
       this.img = img;
       calculate();
    }
    
    /**
     * Calcula os valores de beta1, beta0 e r2 
     * @param img
     */
    private void calculate(){
        double sumX = 0.0;
        double sumY = 0.0;
        double sumX2 = 0.0;
        double sumXY = 0.0;
        Point points[] = getPoints(img);
        int n = points.length;
        
        for(int i=0; i < points.length; i++){
            sumX  += points[i].x;
            sumY  += points[i].y; 
            sumX2 += Math.pow(points[i].x, 2);
            sumXY += points[i].x * points[i].y;
            
        }
        //equacao da reta
        this.beta1 = (n * sumXY - sumX * sumY) / (n * sumX2 - Math.pow(sumX, 2)); //a
        this.beta0 = (sumY - beta1 * sumX) / n; //b

        
        //System.out.println("y = "+ this.beta1 +"x + " + this.beta0);
        
        double xBar = sumX / n;
        double yBar = sumY / n;

        // second pass: compute summary statistics
        double xxBar = 0.0, yyBar = 0.0, xyBar = 0.0;
        for (int i = 0; i < n; i++) {
            xxBar += Math.pow(points[i].x - xBar, 2);
            yyBar += Math.pow(points[i].y - yBar, 2);
            xyBar += (points[i].x - xBar) * (points[i].y - yBar);
        }


        // analyze results
        double ssr = 0.0;      // regression sum of squares
        for (int i = 0; i < n; i++) {
            double fit = this.beta1* points[i].x + this.beta0;
            ssr += (fit - yBar) * (fit - yBar);
        }
        this.r2    = ssr / yyBar;
        //System.out.println("R^2 = " + r2);
    }
    
    /**
     * Pega o conjunto de pontos da imagens
     * @param img
     * @return
     */
    private Point[] getPoints(IBinaryImage img){
        int n = img.getArea();
        Point[] points = new Point[n];
        for(int i=0, count=0; i < img.getWidth(); i++){
            for(int j=0; j < img.getHeight(); j++){
                if(img.getPixel(i, j) == img.getForeground()){
                    points[count++] = new Point(i, j);
                }
            }
        }
        return points;
    }
    
    /**
     * Pega o coeficiente de determinacao
     * @return
     */
    public double r2(){
        return r2;
    }
    
    /**
     * Pega o valor de beta1, isto eh o valor de "a" da equacao y = ax+b
     * @return
     */
    public double getBeta1(){
        return this.beta1;
    }
    
    /**
     * Pega o valor de beta0, isto eh o valor de "b" da equacao y = ax+b
     * @return
     */
    public double getBeta0(){
        return this.beta1;
    }
    
    /**
     * Pega o ponto inicial da reta da regressao
     * @return
     */
    public Point getPointInitial(){
        return new Point(0, (int) this.beta0);
    }
    
    /**
     * Pega o ponto final da reta da regressao
     * @return
     */
    public Point getPointEnd(){
        return new Point(img.getWidth()-1, (int)(this.beta1 * img.getWidth()-1 + this.beta0));
    }
    
    /**
     * Desenha a reta de regressao
     * @param img
     * @return
     */
    public IRGBImage paint(){
        IRGBImage imgOut = new RGBImage(img.getWidth(), img.getHeight());
        imgOut.setPixels(img.getPixels());

        for(int i=0; i < imgOut.getWidth(); i++){
            imgOut.setPixel(i, (int)(this.beta1 * i + this.beta0), Color.RED);
        }
                
        return imgOut;
    }
    
}

